﻿Type.registerNamespace("AspNetAjaxExtensions");

AspNetAjaxExtensions.UpdatePanelIFrameExecutor = function(sourceElement)
{
    AspNetAjaxExtensions.UpdatePanelIFrameExecutor.initializeBase(this);

    // for properties
    this._started = false;
    this._responseAvailable = false;
    this._timedOut = false;
    this._aborted = false;
    this._responseData = null;
    this._statusCode = null;
    
    // the element initiated the async postback
    this._sourceElement = sourceElement;
    // the form in the page.
    this._form = Sys.WebForms.PageRequestManager.getInstance()._form;
}

AspNetAjaxExtensions.UpdatePanelIFrameExecutor.prototype =
{
    get_started : function()
    {
        return this._started
    },
    
    get_responseAvailable : function()
    {
        return this._responseAvailable;
    },

    get_timedOut : function()
    {
        return this._timedOut;
    },
    
    get_aborted : function()
    {
        return this._aborted;
    },
    
    get_responseData : function()
    {
        return this._responseData;
    },
    
    get_statusCode : function()
    {
        return this._statusCode;
    },

    executeRequest : function()
    {
        this._addAdditionalHiddenElements();
    
        var onSuccess = Function.createDelegate(this, this._onSuccess);
        var onFailure = Function.createDelegate(this, this._onFailure);
    
        this._started = true;

        var timeout = this._webRequest.get_timeout();
        if (timeout > 0)
        {
            this._timer = window.setTimeout(Function.createDelegate(this, this._onTimeout), timeout);
        }
        
        AspNetAjaxExtensions.UpdatePanelIFrameExecutor._executeForm(this._form, onSuccess, onFailure);
    },
    
    _addHiddenElement : function(name, value)
    {
        var hidden = document.createElement("input");
        hidden.name = name;
        hidden.value = value;
        hidden.type = "hidden";
        this._form.appendChild(hidden);
        Array.add(this._hiddens, hidden);
    },
    
    _addAdditionalHiddenElements : function()
    {
        var prm = Sys.WebForms.PageRequestManager.getInstance();
        
        this._hiddens = [];
        
        this._addHiddenElement(prm._scriptManagerID, prm._postBackSettings.panelID);
        this._addHiddenElement("__UpdatePanelUploading__", "true");
        
        var additionalInput = null;
        var element = this._sourceElement;
        
        if (element.name)
        {
            var requestBody = this.get_webRequest().get_body();
            var index = -1;
            
            if (element.tagName === 'INPUT')
            {
                var type = element.type;
                if (type === 'submit')
                {
                    index = requestBody.lastIndexOf("&" + element.name + "=");
                }
                else if (type === 'image')
                {
                    index = requestBody.lastIndexOf("&" + element.name + ".x=");
                }
            }
            else if ((element.tagName === 'BUTTON') && (element.name.length !== 0) && (element.type === 'submit'))
            {
                index = requestBody.lastIndexOf("&" + element.name + "=");
            }
            
            if (index > 0)
            {
                additionalInput = requestBody.substring(index + 1);
            }
        }
        
        if (additionalInput)
        {
            var inputArray = additionalInput.split("&");
            for (var i = 0; i < inputArray.length; i++)
            {
                var nameValue = inputArray[i].split("=");
                this._addHiddenElement(nameValue[0], decodeURIComponent(nameValue[1]));
            }
        }
    },
    
    _removeAdditionalHiddenElements : function()
    {
        var hiddens = this._hiddens;
        delete this._hiddens;
        
        for (var i = 0; i < hiddens.length; i++)
        {
            hiddens[i].parentNode.removeChild(hiddens[i]);
        }
        
        hiddens.length = 0;
    },
    
    _onSuccess : function(responseData)
    {
        this._clearTimer();
        if (this._aborted || this._timedOut) return;
        
        this._statusCode = 200;
        this._responseAvailable = true;
        this._responseData = responseData;
        
        this._removeAdditionalHiddenElements();
        this.get_webRequest().completed(Sys.EventArgs.Empty);
    },
    
    _onFailure : function()
    {
        this._clearTimer();
        if (this._aborted || this._timedOut) return;
        
        this._statusCode = 500;
        this._responseAvailable = false;
        
        this._removeAdditionalHiddenElements();
        this.get_webRequest().completed(Sys.EventArgs.Empty);
    },
    
    abort : function()
    {
        this._aborted = true;
        this._clearTimer();
        
        this._removeAdditionalHiddenElements();
    },
    
    _onTimeout : function()
    {
        this._timedOut = true;
        
        this._statusCode = 500;
        this._responseAvailable = false;

        this._removeAdditionalHiddenElements();
        this.get_webRequest().completed(Sys.EventArgs.Empty);
    },
    
    _clearTimer : function()
    {
        if (this._timer != null)
        {
            window.clearTimeout(this._timer);
            delete this._timer;
        }
    }
}
AspNetAjaxExtensions.UpdatePanelIFrameExecutor.registerClass("AspNetAjaxExtensions.UpdatePanelIFrameExecutor", Sys.Net.WebRequestExecutor);

AspNetAjaxExtensions.UpdatePanelIFrameExecutor._beginRequestHandler = function(sender, e)
{
    var inputList = document.getElementsByTagName("input");
    for (var i = 0; i < inputList.length; i++)
    {
        var type = inputList[i].type;
        if (type && type.toUpperCase() == "FILE")
        {
            e.get_request().set_executor(new AspNetAjaxExtensions.UpdatePanelIFrameExecutor(e.get_postBackElement()));
            return;
        }
    }
}

Sys.Application.add_init(function()
{
    Sys.WebForms.PageRequestManager.getInstance().add_beginRequest(
        AspNetAjaxExtensions.UpdatePanelIFrameExecutor._beginRequestHandler);
});
